@@ -3,19 +3,22 @@
 #include <hildon/hildon.h>
 #include <libosso.h>
 #include "scout-window.h"
+#include "calendar-event-preview.hpp"
 #include <rtcom-eventlogger/event.h>
 #include <string.h>
 #include <libintl.h>
 #include <locale.h>
-#include <CalInterface.h>
+//#include <CalInterface.h>
 #include <CMulticalendar.h>
-#include <vector>
 #include <CComponent.h>
-#include <CCalendar.h>
-#include <CJournal.h>
+#include <Common.h>
+#include <vector>
 #include <ctime>
 #include <clockd/libtime.h>
 #include <rtcom-eventlogger-plugins/chat.h>
+#include <rtcom-eventlogger-ui/rtcom-log-model.h>
+#include <rtcom-eventlogger-ui/rtcom-log-view.h>
+#include <rtcom-eventlogger-ui/rtcom-log-columns.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/dbus.h>
 #include <rtcom-telepathy-glib/extensions.h>
@@ -70,8 +73,11 @@ enum
 {
   SEARCH_CATEGORY_CONTACT = 0,
   SEARCH_CATEGORY_CONVERSATION_SMS = 1,
-  SEARCH_CATEGORY_CALENDAR = 2,
-  SEARCH_CATEGORY_CONVERSATION_IM = 3
+  SEARCH_CATEGORY_CONVERSATION_IM = 2,
+  SEARCH_CATEGORY_CALENDAR_EVENT = 3,
+  SEARCH_CATEGORY_CALENDAR_TODO = 4,
+  SEARCH_CATEGORY_CALENDAR_JOURNAL = 5,
+
 };
 
 enum
@@ -299,7 +305,7 @@ _get_event_type_icon(ScoutWindow* self, const gint event_type)
 						 HILDON_ICON_PIXEL_SIZE_SMALL, GTK_ICON_LOOKUP_NO_SVG, NULL);
       break;
     case 1:
-    case E_NORMAL_EVENT:
+
       event_type_icon = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
 						 "general_calendar",
 						 HILDON_ICON_PIXEL_SIZE_SMALL, GTK_ICON_LOOKUP_NO_SVG, NULL);
@@ -636,7 +642,9 @@ _find_in_text(ScoutWindow* self, const gchar* text0, gchar** prefix, gchar** suf
       *prefix = g_strndup(g_utf8_offset_to_pointer(text0, query_offset - prefix_char_len),
 			  pos - g_utf8_offset_to_pointer(text, query_offset - prefix_char_len));
       *suffix = g_strndup(g_utf8_offset_to_pointer(text0, query_offset + query_char_len),
-			  g_utf8_offset_to_pointer(text, query_offset + query_char_len + suffix_char_len) - g_utf8_offset_to_pointer(text,query_offset + query_char_len));
+			  g_utf8_offset_to_pointer(text, 
+						   query_offset + query_char_len + suffix_char_len) - 
+			  g_utf8_offset_to_pointer(text,query_offset + query_char_len));
     }
     g_free(text);
     return TRUE;
@@ -823,112 +831,117 @@ search_in_conversations_sms(ScoutWindow* self)
 		    SEARCH_CATEGORY_CONVERSATION_SMS);
 }
 
+static gchar*
+_format_time(time_t start_time)
+{
+  gchar* time_string = (gchar*)g_malloc0(256);
+  struct tm* time_stamp;
+  time_stamp = localtime(&start_time);
+  strftime(time_string, 256, _get_date_time_format(), time_stamp);
+  return time_string;
+}
+
+static void
+_find_and_insert_calendar_event(ScoutWindow* self, CComponent* component, int cal_id)
+{
+  gchar* prefix = NULL;
+  gchar* suffix = NULL;
+
+  int event_type = component->getType();
+  if(_find_in_text(self, component->getSummary().c_str(), &prefix, &suffix) ||
+     (event_type != 3 && 
+      _find_in_text(self, component->getDescription().c_str(), &prefix, &suffix)))
+  {
+    gchar* title_text = NULL;
+    const gchar* faulty_char = NULL;
+    if(!g_utf8_validate(component->getSummary().c_str(), -1, &faulty_char))
+    {
+      g_warning("not valid %s\n", component->getSummary().c_str());
+      title_text = g_strndup(component->getSummary().c_str(),
+			     faulty_char - component->getSummary().c_str());
+    }
+    else
+    {
+      title_text = g_strdup(component->getSummary().c_str());
+    }
+    gchar* time_string = _format_time(component->getDateStart());
+    gchar* result_text =
+      g_markup_printf_escaped(_get_calendar_markup_template(),
+			      title_text,
+			      time_string,
+			      prefix,
+			      self->priv->current_search_text,
+			      suffix);
+    g_free(title_text);
+    g_free(time_string);
+    int search_category = 0;
+    if(component->getType() == E_EVENT ||
+       component->getType() == E_BDAY)
+    {
+      search_category = SEARCH_CATEGORY_CALENDAR_EVENT;
+    }else if(component->getType() == E_TODO)
+    {
+      search_category = SEARCH_CATEGORY_CALENDAR_TODO;
+    }else if(component->getType() == E_JOURNAL)
+    {
+      search_category = SEARCH_CATEGORY_CALENDAR_JOURNAL;
+    }
+
+
+    _add_result_entry(self,
+		      self->priv->search_category_calendar_icon,
+		      result_text,
+		      _get_event_type_icon(self, event_type),
+		      _get_calendar_id_icon(self, cal_id),
+		      search_category,
+		      component->getId().c_str(),
+		      cal_id);
+		      
+    g_free(result_text);
+  }
+  g_free(prefix);
+  g_free(suffix);
+}
+
 static void
 search_in_calendar(ScoutWindow* self)
 {
   int error = 0;
-  int offset = 0;
-  int size = 0;
-  int limit = 100;
   time_t start_time = -1;
   time_t end_time = 2145909599;
   vector<CCalendar*> calendar_list = CMulticalendar::MCInstance()->getListCalFromMc();
-  GtkTreeIter iter;
-  const gchar* time_format = NULL;
-  const gchar* date_format = NULL;
-  date_format = dgettext("hildon-libs", "wdgt_va_date");
-  time_format = dgettext("hildon-libs", "wdgt_va_24h_time");
-  gchar* full_format = g_strdup_printf("%s | %s", date_format, time_format);
   for(int j=0;self->priv->search_active && j<calendar_list.size();++j)
   {
+    int offset = 0;
+    int limit = 100;
     vector<CComponent*> component_list = calendar_list[j]->getAllEventsTodos(start_time,
 									     end_time,
 									     limit,
 									     offset,
 									     error);
-    for(int i = 0;i<component_list.size();++i)
+    while(component_list.size()>0)
     {
-      gchar* prefix = NULL;
-      gchar* suffix = NULL;
-
-      const gchar* text = NULL;
-      if(_find_in_text(self, component_list[i]->getDescription().c_str(), &prefix, &suffix) ||
-	 _find_in_text(self, component_list[i]->getSummary().c_str(), &prefix, &suffix))
+      offset += component_list.size();
+      for(int i = 0;i<component_list.size();++i)
       {
-	int event_type = component_list[i]->getType();
-	GdkPixbuf* event_type_icon = _get_event_type_icon(self, event_type);
-	GdkPixbuf* calendar_id_icon = _get_calendar_id_icon(self, component_list[i]->getCalendarId());
-      
-	gchar time_string[256];
-	struct tm* time_stamp;
-	time_t start_time = component_list[i]->getDateStart();
-	time_stamp = localtime(&start_time);
-	strftime(time_string, 256, full_format, time_stamp);
-	gchar* text_name = g_markup_printf_escaped(_get_calendar_markup_template(),
-						   component_list[i]->getSummary().c_str(), 
-						   time_string,
-						   prefix,
-						   self->priv->current_search_text,
-						   suffix);
-
-	_add_result_entry(self,
-			  self->priv->search_category_calendar_icon,
-			  text_name,
-			  event_type_icon,
-			  calendar_id_icon,
-			  SEARCH_CATEGORY_CALENDAR,
-			  NULL,
-			  component_list[i]->getDateStart());
-	g_free(text_name);
-	g_free(prefix);
-	g_free(suffix);
+	_find_and_insert_calendar_event(self, component_list[i], calendar_list[j]->getCalendarId());
+	delete component_list[i];
       }
-      delete component_list[i];
+      component_list = calendar_list[j]->getAllEventsTodos(start_time,
+							   end_time,
+							   limit,
+							   offset,
+							   error);
     }
+
     vector<CJournal*> journal_list = calendar_list[j]->getJournalListDetailed(error);
     for(int i = 0;i<journal_list.size();++i)
     {
-      gchar* prefix = NULL;
-      gchar* suffix = NULL;
-
-      const gchar* text = NULL;
-      if(_find_in_text(self, journal_list[i]->getSummary().c_str(), &prefix, &suffix))
-      {
-	int event_type = journal_list[i]->getType();
-	GdkPixbuf* event_type_icon = _get_event_type_icon(self, event_type);
-	GdkPixbuf* calendar_id_icon = _get_calendar_id_icon(self, calendar_list[j]->getCalendarId());
-      
-	gchar time_string[256];
-	struct tm* time_stamp;
-	time_t start_time = journal_list[i]->getDateStart();
-	time_stamp = localtime(&start_time);
-	strftime(time_string, 256, full_format, time_stamp);
-	gchar* text_name = g_markup_printf_escaped(_get_calendar_markup_template(),
-						   journal_list[i]->getSummary().c_str(), 
-						   time_string,
-						   prefix,
-						   self->priv->current_search_text,
-						   suffix);
-
-	_add_result_entry(self,
-			  self->priv->search_category_calendar_icon,
-			  text_name,
-			  event_type_icon,
-			  calendar_id_icon,
-			  SEARCH_CATEGORY_CALENDAR,
-			  NULL,
-			  journal_list[i]->getDateStart());
-
-	g_free(text_name);
-	g_free(prefix);
-	g_free(suffix);
-      }
+      _find_and_insert_calendar_event(self, journal_list[i], calendar_list[j]->getCalendarId());
       delete journal_list[i];
     }
-
   }
   CMulticalendar::MCInstance()->releaseListCalendars(calendar_list);
-  g_free(full_format);
 }
 
 static gboolean
@@ -949,7 +962,6 @@ _search_and_insert_contact_attribute(ScoutWindow* self, OssoABookContact* contac
     if(value && 
        _find_in_text(self, value, &prefix, &suffix))
     {
-      GtkTreeIter iter;
       const gchar* abook_uid = osso_abook_contact_get_persistent_uid(contact);
       GdkPixbuf* avatar_icon = _get_contact_avatar_icon(self, abook_uid);
       GdkPixbuf* attribute_icon = _get_contact_attribute_icon(self, e_contact_field);
@@ -1136,9 +1148,90 @@ on_search_clicked(GtkButton* button, gpointer user_data)
   }
 }
 
+typedef struct _event_logger_window_data_t
+{
+  GtkWidget* view;
+  RTComLogModel* model;
+  GtkWidget* text_view;
+  GtkWidget* window;
+  int event_id;
+  ScoutWindow* scout;
+} event_logger_window_data_t;
+
+static void
+_scroll_to_event(GtkTreeModel *tree_model,
+		 GtkTreePath  *path,
+		 GtkTreeIter  *iter,
+		 gpointer      user_data)
+{
+  event_logger_window_data_t* event_logger_window_data = (event_logger_window_data_t*)user_data;
+  int event_id = 0;
+  gtk_tree_model_get(tree_model,
+		     iter,
+		     RTCOM_LOG_VIEW_COL_EVENT_ID, &event_id,
+		     -1);
+  if(event_logger_window_data->event_id == event_id)
+  {
+    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(event_logger_window_data->view),
+				 path, NULL,
+				 TRUE,
+				 0, 0);
+    gchar* text = NULL;
+    gtk_tree_model_get(tree_model, iter,
+		       RTCOM_LOG_VIEW_COL_TEXT, &text,
+		       -1);
+    
+    GtkTextBuffer* buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(event_logger_window_data->text_view));
+    gtk_text_buffer_insert_at_cursor(buffer,
+				     text,
+				     strlen(text));
+    g_free(text);
+    hildon_gtk_window_set_progress_indicator(GTK_WINDOW(event_logger_window_data->window), 0);
+  }
+}
+
+static void
+_delete_event_scroller(GtkWidget* widget,
+		       GdkEvent* event,
+		       gpointer user_data)
+{
+  event_logger_window_data_t* event_logger_window_data  = (event_logger_window_data_t*)user_data;
+  g_object_unref(event_logger_window_data->model);
+  g_free(event_logger_window_data);
+}
+
+static void
+_show_current_text(GtkTreeView* tree_view, 
+		   GtkTreePath* path, 
+		   GtkTreeViewColumn* column,
+		   gpointer data)
+{
+  GtkTreeSelection* selection = gtk_tree_view_get_selection(tree_view);
+  GtkTreeIter iter;
+  GtkTextView* text_view = GTK_TEXT_VIEW(data);
+  if(gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_tree_view_get_model(tree_view)),
+			     &iter,
+			     path))
+  {
+    gchar* text = NULL;
+    gtk_tree_model_get(GTK_TREE_MODEL(gtk_tree_view_get_model(tree_view)), &iter,
+		       RTCOM_LOG_VIEW_COL_TEXT, &text,
+		       -1);
+    GtkTextBuffer* buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
+    GtkTextIter start;
+    GtkTextIter end;
+    gtk_text_buffer_get_start_iter(buffer, &start);
+    gtk_text_buffer_get_end_iter(buffer, &end);
+    gtk_text_buffer_delete(buffer, &start, &end);
+    gtk_text_buffer_insert_at_cursor(buffer,
+				     text,
+				     strlen(text));
+    g_free(text);
+  }
+}
+
 static void
 scout_open_conversation_app(ScoutWindow* self,
-			    const gchar* user_id,
 			    int event_id)
 {
   RTComElQuery *query = rtcom_el_query_new (self->priv->rt_el);
@@ -1224,6 +1317,95 @@ scout_open_conversation_app(ScoutWindow* self,
   }
 }
 
+static void
+open_conversation_button_cb(GtkButton* button, gpointer user_data)
+{  
+  event_logger_window_data_t* event_logger_window_data  = (event_logger_window_data_t*)user_data;
+  scout_open_conversation_app(event_logger_window_data->scout,
+			      event_logger_window_data->event_id);
+}
+
+static void
+scout_open_event_log_window(ScoutWindow* self,
+			    const gchar* user_id,
+			    int event_id)
+{
+  GtkWidget* window;
+  window = hildon_stackable_window_new();
+  const gchar* services[] = {"RTCOM_EL_SERVICE_CHAT", "RTCOM_EL_SERVICE_SMS", NULL};
+
+  RTComElQuery *query = rtcom_el_query_new (self->priv->rt_el);
+  rtcom_el_query_prepare (query, "remote-uid", user_id, RTCOM_EL_OP_EQUAL, 
+			  "service", services, RTCOM_EL_OP_IN_STRV, 
+			  "id", event_id, RTCOM_EL_OP_EQUAL,
+			  NULL);
+  RTComElIter* rtcom_iter = rtcom_el_get_events(self->priv->rt_el, query);
+  if(rtcom_el_iter_first(rtcom_iter))
+  {
+    int start_time = 0;
+    int seven_days = 7 * 24 * 60 * 60;
+    rtcom_el_iter_get_values(rtcom_iter, "start-time", &start_time, NULL);
+    g_object_unref(query);
+    g_object_unref(rtcom_iter);
+    query = rtcom_el_query_new (self->priv->rt_el);    
+    rtcom_el_query_prepare (query, 
+			    "remote-uid", user_id, RTCOM_EL_OP_EQUAL, 
+			    "service", services, RTCOM_EL_OP_IN_STRV, 
+			    "start-time", start_time - seven_days, RTCOM_EL_OP_GREATER_EQUAL,
+			    "start-time", start_time + seven_days, RTCOM_EL_OP_LESS_EQUAL,
+			    NULL);
+  }
+  else
+  {
+    g_object_unref(query);
+    query = rtcom_el_query_new (self->priv->rt_el);    
+    rtcom_el_query_prepare (query, 
+			    "remote-uid", user_id, RTCOM_EL_OP_EQUAL, 
+			    "service", services, RTCOM_EL_OP_IN_STRV, 
+			    NULL);
+
+  }
+  event_logger_window_data_t* event_logger_window_data = g_new0(event_logger_window_data_t,1);
+  RTComLogModel* rtcom_model = rtcom_log_model_new();
+  GtkWidget* rtcom_log_view = rtcom_log_view_new();
+  GtkWidget* text_view = hildon_text_view_new();
+  GtkWidget* open_conversation_button = 
+    hildon_button_new_with_text(HILDON_SIZE_FINGER_HEIGHT,
+				HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+				dgettext("rtcom-messaging-ui", "messaging_ap_conversations"), 
+				NULL);
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_CHAR);
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
+
+  event_logger_window_data->view = rtcom_log_view;
+  event_logger_window_data->model = rtcom_model;
+  event_logger_window_data->window = window;
+  event_logger_window_data->text_view = text_view;
+  event_logger_window_data->event_id = event_id;
+  event_logger_window_data->scout = self;
+  rtcom_log_model_set_abook_aggregator(rtcom_model, OSSO_ABOOK_AGGREGATOR(self->priv->abook));
+  rtcom_log_view_set_model(RTCOM_LOG_VIEW(rtcom_log_view),
+			   GTK_TREE_MODEL(rtcom_model));
+  g_signal_connect(rtcom_model, "row-inserted", G_CALLBACK(_scroll_to_event), event_logger_window_data);
+  g_signal_connect(open_conversation_button, "clicked", G_CALLBACK(open_conversation_button_cb), event_logger_window_data);
+  GtkWidget* pan = hildon_pannable_area_new();
+  GtkWidget* content = gtk_vbox_new(FALSE, 0);
+  GtkWidget* content2 = gtk_vbox_new(TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(pan), rtcom_log_view);
+  gtk_box_pack_start(GTK_BOX(content), text_view, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(content), open_conversation_button, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(content2), pan, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(content2), content, TRUE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(window), content2);
+  gtk_window_set_title(GTK_WINDOW(window), user_id);
+  hildon_gtk_window_set_progress_indicator(GTK_WINDOW(window), 1);
+  rtcom_log_model_populate_query(rtcom_model, query);
+  g_object_unref(query);
+  g_signal_connect(rtcom_log_view, "row-activated", G_CALLBACK(_show_current_text), text_view);
+  g_signal_connect(window, "destroy-event", G_CALLBACK(_delete_event_scroller), event_logger_window_data);
+  gtk_widget_show_all(window);
+}
+
 static void
 scout_open_text_message_window(ScoutWindow* self,
 			       const gchar* user_id,
@@ -1262,23 +1444,7 @@ scout_open_text_message_window(ScoutWindow* self,
   gtk_widget_show_all(window);
 }
 
-static void
-_open_calendar(ScoutWindow* self, int start_time)
-{
-  int type = 3;
-  const char* text = "DUMMY";  
 
-  osso_rpc_run(self->priv->osso,
-	       CALENDAR_SERVICE, 
-	       CALENDAR_PATH, 
-	       CALENDAR_INTERFACE,
-	       CALENDAR_LAUNCH_METHOD,
-	       NULL,
-	       DBUS_TYPE_UINT32, type,
-	       DBUS_TYPE_INT32, start_time, 
-	       DBUS_TYPE_STRING, text,
-	       DBUS_TYPE_INVALID);
-}
 
 static void 
 open_result_from_list(GtkTreeView* tree_view, 
@@ -1295,13 +1461,15 @@ open_result_from_list(GtkTreeView* tree_view,
   {
     gchar* name = NULL;
     gchar* id = NULL;
+    gchar* string_id = NULL;
     int type;
-    int event_id;
+    int int_id;
     gtk_tree_model_get(GTK_TREE_MODEL(self->priv->result_list), &iter,
 		       SEARCH_RESULT_TEXT_COL, &name,
 		       SEARCH_RESULT_ITEM_TYPE_COL, &type,
 		       SEARCH_RESULT_ITEM_STRING_ID_COL, &id,
-		       SEARCH_RESULT_ITEM_INT_ID_COL, &event_id,
+		       SEARCH_RESULT_ITEM_INT_ID_COL, &int_id,
+		       SEARCH_RESULT_ITEM_STRING_ID_COL, &string_id,
 		       -1);
 
     if(type == SEARCH_CATEGORY_CONTACT)
@@ -1318,17 +1486,27 @@ open_result_from_list(GtkTreeView* tree_view,
       }
       g_list_free(entries);
     }
-    else if(type == SEARCH_CATEGORY_CALENDAR)
+    else if(type == SEARCH_CATEGORY_CALENDAR_EVENT)
     {
-      _open_calendar(self, event_id);
+      open_calendar_event_window(self->priv->osso, int_id, string_id);
+    }
+    else if(type == SEARCH_CATEGORY_CALENDAR_TODO)
+    {
+      open_calendar_todo_window(self->priv->osso, int_id, string_id);
+    }
+    else if(type == SEARCH_CATEGORY_CALENDAR_JOURNAL)
+    {
+      open_calendar_journal_window(self->priv->osso, int_id, string_id);
     }
     else if(type == SEARCH_CATEGORY_CONVERSATION_SMS ||
 	    type == SEARCH_CATEGORY_CONVERSATION_IM)
     {
-      scout_open_conversation_app(self, id, event_id);
+      //      scout_open_conversation_app(self, id, event_id);
       //      scout_open_text_message_window(self, id, event_id);
+      scout_open_event_log_window(self, id, int_id);
     }
     g_free(name);
+    g_free(string_id);
     g_free(id);
     gtk_tree_selection_unselect_all(selection);
   }
@@ -1368,7 +1546,6 @@ _read_settings(ScoutWindow* self)
 static void 
 _store_settings(ScoutWindow* self)
 {
-  g_warning("store settings\n");
   GConfClient* client = NULL;
   client = gconf_client_get_default();
   g_return_if_fail(GCONF_IS_CLIENT(client));
